/*
 * $Id: apc.c,v 2.0.1.1 1996/01/30 09:17:09 alexis Exp alexis $
 *
 * UPS Daemon
 * The Wild Wind Communications, 1995, 1996
 *
 * See file LICENSE for the distribution terms of this software.
 */

#include "upsd.h"
#include "apc.h"

/*
 * Polls APC SmartUPS value.
 *
 * Returns -1 on error, 1 if timed out or 0 if succeeded.
 */
int
apc_poll(val)
	struct ups_val *val;
{
	register int s;
	char *t;
	struct ups_reg *r;

	if((r = REGISTERID(val->id)) == NULL) {
		syslog(LOG_ERR, "apc_poll: incorrect register id");
		return -1;
	}
	if((t = alloca(POLLBUFLEN)) == NULL) {
		syslog(LOG_ERR, "apc_poll: alloca failed: %m");
		return -1;
	}

	for(;;) {
		flushport();

		if((s = writeport(r->cmd_data, r->cmd_size)) != r->cmd_size) {
			return -1;	/* Can't write to the port. */
		}

		if(r->size == 0) {
			return 0;	/* We don't expect anything back. */
		}

		if((s = readport(t, r->size, 1)) == -1) {
			return -1;	/* Can't read the port. */
		};

		if(s != r->size) {
			return 1;	/* Timed out on readport. */
		};

		switch(r->type) {
			case T_BINARY:
				bcopy(t, val->val.binary, (size_t) r->size - r->prec);
				((char *)val->val.binary)[r->size - r->prec] = '\0'; /* XXX: correct? */
				return 0;
				/* NOTREACHED */
			case T_HEX:
				val->val.number = (double)strtol(t, (char **)NULL, 16);
				return 0;
				/* NOTREACHED */
			case T_DEC:
				val->val.number = (double)strtol(t, (char **)NULL, 10);
				return 0;
				/* NOTREACHED */
			case T_OCT:
				val->val.number = (double)strtol(t, (char **)NULL, 8);
				return 0;
				/* NOTREACHED */
			case T_NUMBER:
				val->val.number = strtod(t, (char **)NULL);
				return 0;
				/* NOTREACHED */
			default:
				syslog(LOG_ERR, "apc_poll: incorrect value type");
				return -1;
				/* NOTREACHED */
		}
		/* NOTREACHED */
	}
	/* NOTREACHED */
	return -1;
}

/*
 * Tunes the value of APC SmartUPS.
 *
 * Returns -1 upon error, 1 if the string was not toggled, and zero
 * if succeeded.
 */
int
apc_tune(val)
	struct ups_val *val;
{
	syslog(LOG_INFO, "apc_tune is not implemented yet");
	return 0;
}
